@@ -10,6 +10,10 @@ use std::fs;
 use std::io::ErrorKind;
 use tiny_http::{Header, Request, Response, StatusCode};
 
+#[allow(non_snake_case)]
+mod isBin;
+
+#[allow(non_snake_case)]
 pub fn serveFile(rq: Request, path: &str, cors: &str) -> Result<(), Error> {
     //println!("{}", cors);
 
@@ -56,10 +60,14 @@ pub fn serveFile(rq: Request, path: &str, cors: &str) -> Result<(), Error> {
         }
     }
 
+    let isplaintext = !isBin::isBin("./".to_owned() + &path);
     let kind = infer::get_from_path("./".to_owned() + &path).unwrap();
     match kind {
         Some(value) => {
-            let kind = value.mime_type();
+            let mut kind = value.mime_type();
+            if isplaintext {
+                kind = "text/plain";
+            }
             let rs = rs.unwrap();
             let length = fs::metadata(&path).unwrap().len();
             let result = rq.respond(
@@ -80,12 +88,16 @@ pub fn serveFile(rq: Request, path: &str, cors: &str) -> Result<(), Error> {
             return result;
         }
         None => {
+	    let mut kind = "application/octet-stream";
+            if isplaintext {
+                kind = "text/plain";
+            }
             let rs = rs.unwrap();
             let result = rq.respond(
                 Response::from_file(rs)
                     .with_status_code(StatusCode(200))
                     .with_header(
-                        Header::from_bytes(&b"Content-Type"[..], &b"application/octet-stream"[..])
+                        Header::from_bytes(&b"Content-Type"[..], kind.as_bytes())
                             .unwrap(),
                     ),
             );
